/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editor.fx.visual;

import static org.fortiss.tooling.base.ui.utils.LWFXEditorUtils.stickyConnectorHasVisibleConnections;

import org.fortiss.tooling.base.model.element.IConnector;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;
import org.fortiss.tooling.common.ui.javafx.lwfxef.DiagramLayers;
import org.fortiss.tooling.common.ui.javafx.lwfxef.mvc.IContentAnchorageMVCBundle;
import org.fortiss.tooling.common.ui.javafx.lwfxef.visual.IVisual;
import org.fortiss.tooling.kernel.model.INamedElement;

import javafx.geometry.Rectangle2D;
import javafx.geometry.VPos;
import javafx.scene.text.Text;
import javafx.scene.text.TextAlignment;

/**
 * {@link IVisual} for layouted and named {@link IConnector}s of some content element within a
 * diagram (sticky connectors).
 * 
 * @author diewald
 */
public class NamedLayoutedCircularAnchorageContentVisual<T extends ILayoutedModelElement & INamedElement & IConnector>
		extends LayoutedCircularAnchorageContentVisualBase<T> {
	/** The name text label. */
	private final Text nameText = new Text("");

	/** Constructor. */
	public NamedLayoutedCircularAnchorageContentVisual(IContentAnchorageMVCBundle mvcb,
			Class<T> modelType) {
		super(mvcb, modelType);
		nameText.setMouseTransparent(true);
		nameText.setTextAlignment(TextAlignment.LEFT);
		nameText.setTextOrigin(VPos.CENTER);
	}

	/** {@inheritDoc} */
	@Override
	public void updateNodes(DiagramLayers layers) {
		super.updateNodes(layers);
		if(enableName()) {
			Rectangle2D bounds = getCurrentBounds();
			nameText.setX(bounds.getMaxX());
			nameText.setY(bounds.getMinY() + bounds.getHeight() / 2);
			nameText.setText(getName());
			if(nameText.getParent() == null) {
				layers.getVisualFeedbackLayer().add(nameText, getMVCBundle());
			}
		} else if(nameText.getParent() != null) {
			layers.getVisualFeedbackLayer().remove(nameText);
		}
	}

	/** Returns whether the name label should be enabled. */
	public boolean enableName() {
		return !stickyConnectorHasVisibleConnections(getModelElement());
	}

	/** Returns the name. */
	public String getName() {
		return getModelElement().getName();
	}
}
