/*-------------------------------------------------------------------------+
| Copyright 2011 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editpart.command;

import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;
import org.fortiss.tooling.base.ui.editpart.StickyConnectorEditPartBase;

/**
 * A command to move the sticky connectors within an element.
 * 
 * @author wandinger
 * @author hoelzl
 */
public class MoveWithinElementCommand extends ChangeBoundsRequestCommandBase {

	/** Constructor. */
	public MoveWithinElementCommand(ChangeBoundsRequest request) {
		super(request);
	}

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public void execute() {
		for(EditPart editpart : (List<EditPart>)getRequest().getEditParts()) {
			if(editpart instanceof StickyConnectorEditPartBase) {
				StickyConnectorEditPartBase<? extends ILayoutedModelElement> connectorEditPart =
						(StickyConnectorEditPartBase<? extends ILayoutedModelElement>)editpart;
				connectorEditPart.getLayoutConfiguration()
						.performStickyConnectorMove(connectorEditPart, request);
			}
		}
	}
}
