/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.migration;

import static java.util.Collections.emptyMap;
import static org.fortiss.tooling.base.utils.AnnotationUtils.instantiateAnnotationsRecursive;
import static org.fortiss.tooling.base.utils.AnnotationUtils.requiresAnnotationInstantiation;
import static org.fortiss.tooling.kernel.utils.EcoreUtils.getFirstChildWith;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Map;
import java.util.function.Predicate;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.xml.type.AnyType;
import org.fortiss.tooling.base.model.element.IModelElement;
import org.fortiss.tooling.kernel.extension.IMigrationProvider;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;

/**
 * Adds missing annotations to {@link IModelElement}s.
 * 
 * @author diewald
 */
public class AddMissingAnnotationsMigrationProvider implements IMigrationProvider {

	/** Remembers the processed {@link ITopLevelElement}s. */
	private Collection<ITopLevelElement> migratedProjects = new ArrayList<>();

	/** {@inheritDoc} */
	@Override
	public boolean needMigration(ITopLevelElement modelElement,
			Map<EObject, AnyType> unknownFeatures) {
		if(migratedProjects.contains(modelElement)) {
			return false;
		}

		Predicate<EObject> annotationTest = eObj -> {
			if(eObj instanceof IModelElement &&
					requiresAnnotationInstantiation((IModelElement)eObj)) {
				return true;
			}
			return false;
		};

		if(getFirstChildWith(modelElement.getRootModelElement(), annotationTest) != null) {
			return true;
		}
		return false;
	}

	/** {@inheritDoc} */
	@Override
	public Map<EObject, AnyType> migrate(ITopLevelElement modelElement,
			Map<EObject, AnyType> unknownFeatures) {
		EObject rootElement = modelElement.getRootModelElement();
		instantiateAnnotationsRecursive(rootElement);
		migratedProjects.add(modelElement);
		return emptyMap();
	}
}
