/*******************************************************************************
 * Copyright (c) 2016, 2018 fortiss GmbH.
 * 
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v2.0 which is available at
 * http://www.eclipse.org/legal/epl-v20.html
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *     Florian Hoelzl (fortiss GmbH) - initial implementation
 *     Alexander Diewald (fortiss GmbH) - backport to AF3 & Integration
 *                                        improvements
 *
 *******************************************************************************/
package org.fortiss.tooling.common.ui.javafx.control.treetableview;

import java.util.Collection;
import java.util.HashMap;

import javafx.scene.control.TreeItem;

/**
 * {@link TreeItem} with support for dynamic children using the
 * {@link DynamicList#update()} method.
 */
public class DynamicList<T> extends DynamicTreeItemBase<T> {
	/** The viewer. */
	private DynamicTreeContentProviderBase<T> contentProvider;

	/** Constructor. */
	@SuppressWarnings("unchecked")
	public DynamicList(Collection<T> data, DynamicTreeContentProviderBase<T> contentProvider) {
		super((T)data);
		this.contentProvider = contentProvider;
	}

	/** {@inheritDoc} */
	@Override
	public void update() {
		// remember expanded state of children
		HashMap<T, TreeItem<T>> expanded = new HashMap<>();
		for(TreeItem<T> c : getChildren()) {
			if(c.isExpanded()) {
				expanded.put(c.getValue(), c);
			}
		}
		// get list of children and create tree items
		getChildren().clear();
		for(T element : contentProvider.getFilteredSortedChildren(getValue())) {
			DynamicTreeItem<T> dti = new DynamicTreeItem<T>(element, contentProvider);
			getChildren().add(dti);
			dti.update();
		}
	}

	/** {@inheritDoc} */
	@Override
	public boolean isLeaf() {
		Collection<? extends Object> children = contentProvider.getChildren(getValue());
		return children == null || children.isEmpty();
	}
}
