/*******************************************************************************
 * Copyright (c) 2019, 2020 fortiss GmbH. 
 * 
 * This program and the accompanying materials are made available under the
 * terms of the Apache License, Version 2.0 which is available at
 * https://www.apache.org/licenses/LICENSE-2.0.
 *
 * SPDX-License-Identifier: Apache-2.0
 *******************************************************************************/
package org.fortiss.tooling.common.ui.javafx.lwfxef;

import javafx.geometry.Bounds;
import javafx.geometry.Point2D;
import javafx.scene.Node;

/** Extension of Point2D to improve code understandability when working with coordinates. */
public final class DiagramCoordinate extends Point2D {
	/** Constructor. */
	public DiagramCoordinate(double x, double y) {
		super(x, y);
	}

	/** Returns the location of this diagram coordinate relative to the given node. */
	public Point2D getLocal(Node node) {
		Bounds boundsInLocal = node.getBoundsInLocal();
		return new Point2D(getX() - boundsInLocal.getMinX(), getY() - boundsInLocal.getMinY());
	}

	/** Returns the location of this diagram X coordinate relative to the given node. */
	public double getLocalX(Node node) {
		return getX() - node.getBoundsInLocal().getMinX();
	}

	/** Returns the location of this diagram Y coordinate relative to the given node. */
	public double getLocalY(Node node) {
		return getY() - node.getBoundsInLocal().getMinY();
	}

	/** {@inheritDoc} */
	@Override
	public DiagramCoordinate add(double dx, double dy) {
		return new DiagramCoordinate(getX() + dx, getY() + dy);
	}

	/** {@inheritDoc} */
	@Override
	public DiagramCoordinate subtract(double dx, double dy) {
		return add(-dx, -dy);
	}

	/** {@inheritDoc} */
	@Override
	public DiagramCoordinate subtract(Point2D point) {
		return subtract(point.getX(), point.getY());
	}
}
