/*******************************************************************************
 * Copyright (c) 2019, 2020 fortiss GmbH. 
 * 
 * This program and the accompanying materials are made available under the
 * terms of the Apache License, Version 2.0 which is available at
 * https://www.apache.org/licenses/LICENSE-2.0.
 *
 * SPDX-License-Identifier: Apache-2.0
 *******************************************************************************/
package org.fortiss.tooling.common.ui.javafx.lwfxef.controller;

import org.fortiss.tooling.common.ui.javafx.lwfxef.DiagramCoordinate;
import org.fortiss.tooling.common.ui.javafx.lwfxef.change.Change;

import javafx.scene.Node;
import javafx.scene.input.MouseEvent;

/**
 * Interface for controllers, which handle click gestures provided by the visuals. Each operation
 * may result in {@link Change} to the underlying model.
 */
public interface IClickController {
	/** Handles a single-click at the given mouse location. */
	Change singleClick(MouseEvent event, Node node, DiagramCoordinate lastMouseLocation);

	/** Handles a double-click at the given mouse location. */
	Change doubleClick(MouseEvent event, Node node, DiagramCoordinate lastMouseLocation);

	/**
	 * Handles a single click at the given mouse location when the secondary mouse button is used.
	 */
	Change secondaryClick(MouseEvent event, Node node, DiagramCoordinate lastMouseLocation);
}
