/*******************************************************************************
 * Copyright (c) 2019, 2020 fortiss GmbH. 
 * 
 * This program and the accompanying materials are made available under the
 * terms of the Apache License, Version 2.0 which is available at
 * https://www.apache.org/licenses/LICENSE-2.0.
 *
 * SPDX-License-Identifier: Apache-2.0
 *******************************************************************************/
package org.fortiss.tooling.common.ui.javafx.lwfxef.controller.base;

import org.fortiss.tooling.common.ui.javafx.lwfxef.DiagramCoordinate;
import org.fortiss.tooling.common.ui.javafx.lwfxef.FeedbackChange;
import org.fortiss.tooling.common.ui.javafx.lwfxef.change.Change;
import org.fortiss.tooling.common.ui.javafx.lwfxef.mvc.IContentAnchorageMVCBundle;
import org.fortiss.tooling.common.ui.javafx.lwfxef.mvc.IMVCBundle;

import javafx.geometry.Point2D;
import javafx.geometry.Rectangle2D;
import javafx.scene.Node;

/** Base class for move controllers of content anchorages. */
public abstract class ContentAnchorageMoveControllerBase extends DragControllerBase {
	/** The anchorage. */
	protected final IContentAnchorageMVCBundle anchorage;

	/** Constructor. */
	public ContentAnchorageMoveControllerBase(IContentAnchorageMVCBundle anchorage) {
		this.anchorage = anchorage;
	}

	/** Returns the location of the anchorage when dragged by the given delta. */
	protected abstract Point2D getAnchorageFeedbackLocation(DiagramCoordinate delta);

	/** Sets the angle of the given anchorage when move operation has completed. */
	protected abstract void moveAnchorage(DiagramCoordinate delta);

	/** {@inheritDoc} */
	@Override
	public final void safeDragInProgress(IMVCBundle bundle, Node node,
			DiagramCoordinate diagramLocation) {
		DiagramCoordinate delta = diagramLocation.subtract(startDiagramLocation);
		Point2D c = getAnchorageFeedbackLocation(delta);
		Rectangle2D anchorageBounds = anchorage.getVisual().getModelBounds();
		FeedbackChange change = new FeedbackChange(c.getX() - anchorageBounds.getMinX(),
				c.getY() - anchorageBounds.getMinY(), 0, 0);
		anchorage.setFeedbackChange(change);
	}

	/** {@inheritDoc} */
	@Override
	public final Change safeDragCompleted(IMVCBundle bundle, Node node,
			DiagramCoordinate diagramLocation) {
		DiagramCoordinate delta = diagramLocation.subtract(startDiagramLocation);
		anchorage.setFeedbackChange(FeedbackChange.EMPTY);
		bundle.getViewer().updateVisual(bundle);
		return () -> moveAnchorage(delta);
	}
}
