/*******************************************************************************
 * Copyright (c) 2019, 2020 fortiss GmbH. 
 * 
 * This program and the accompanying materials are made available under the
 * terms of the Apache License, Version 2.0 which is available at
 * https://www.apache.org/licenses/LICENSE-2.0.
 *
 * SPDX-License-Identifier: Apache-2.0
 *******************************************************************************/
package org.fortiss.tooling.common.ui.javafx.lwfxef.mvc.impl;

import static java.util.Objects.requireNonNull;

import java.util.LinkedList;
import java.util.List;

import org.fortiss.tooling.common.ui.javafx.lwfxef.DiagramViewer;
import org.fortiss.tooling.common.ui.javafx.lwfxef.mvc.IAnchorageMVCBundle;
import org.fortiss.tooling.common.ui.javafx.lwfxef.mvc.ILinkMVCBundle;
import org.fortiss.tooling.common.ui.javafx.lwfxef.mvc.IMVCBundle;

/** Base class for {@link IAnchorageMVCBundle}s. */
abstract class AnchorageMVCBundleBase extends MVCBundleBase implements IAnchorageMVCBundle {
	/** The parent bundle this bundle is attached to. */
	private IMVCBundle attachedTo;
	/** The list of incoming links. */
	private final List<ILinkMVCBundle> incomingLinks = new LinkedList<>();
	/** The list of outgoing links. */
	private final List<ILinkMVCBundle> outgoingLinks = new LinkedList<>();

	/** Constructor. */
	public AnchorageMVCBundleBase(Object m, IMVCBundle attachedTo, DiagramViewer v) {
		super(m, v);
		this.attachedTo = requireNonNull(attachedTo);
	}

	/** {@inheritDoc} */
	@Override
	public IMVCBundle getAttachedTo() {
		return attachedTo;
	}

	/** {@inheritDoc} */
	@Override
	public List<ILinkMVCBundle> getIncomingLinks() {
		return incomingLinks;
	}

	/** {@inheritDoc} */
	@Override
	public List<ILinkMVCBundle> getOutgoingLinks() {
		return outgoingLinks;
	}

	/** {@inheritDoc} */
	@Override
	public void changed(Object element) {
		// Anchorages have no children, so only the properties can have changed.
		// Therefore, update just the view and the connected links.
		getViewer().updateVisual(this);
	}
}
