/*******************************************************************************
 * Copyright (c) 2019, 2020 fortiss GmbH. 
 * 
 * This program and the accompanying materials are made available under the
 * terms of the Apache License, Version 2.0 which is available at
 * https://www.apache.org/licenses/LICENSE-2.0.
 *
 * SPDX-License-Identifier: Apache-2.0
 *******************************************************************************/
package org.fortiss.tooling.common.ui.javafx.lwfxef.mvc.impl;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import org.fortiss.tooling.common.ui.javafx.lwfxef.DiagramViewer;
import org.fortiss.tooling.common.ui.javafx.lwfxef.mvc.IContentAnchorageMVCBundle;
import org.fortiss.tooling.common.ui.javafx.lwfxef.mvc.IContentMVCBundle;
import org.fortiss.tooling.common.ui.javafx.lwfxef.mvc.IContentMVCBundleWithParent;
import org.fortiss.tooling.common.ui.javafx.lwfxef.mvc.IDiagramMVCBundle;
import org.fortiss.tooling.common.ui.javafx.lwfxef.visual.IContentVisual;

/** Implementation of {@link IContentMVCBundle}. */
public final class ContentMVCBundle extends MVCBundleBase implements IContentMVCBundleWithParent {
	/** The diagram bundle this content is attached to. */
	private final IDiagramMVCBundle diagramBundle;
	/** The list of attached anchorages. */
	private final List<IContentAnchorageMVCBundle> anchorages = new ArrayList<>();
	/** The parent bundle. */
	private IContentMVCBundle parent;
	/** The child bundles. */
	private final List<IContentMVCBundleWithParent> children =
			new LinkedList<IContentMVCBundleWithParent>();

	/** Constructor. */
	public ContentMVCBundle(Object m, IDiagramMVCBundle diagramBundle, DiagramViewer v) {
		super(m, v);
		this.diagramBundle = diagramBundle;
	}

	/** {@inheritDoc} */
	@Override
	public void remove() {
		// nothing to do here
	}

	/** {@inheritDoc} */
	@Override
	public IDiagramMVCBundle getDiagramBundle() {
		return diagramBundle;
	}

	/** {@inheritDoc} */
	@Override
	public List<IContentAnchorageMVCBundle> getAnchorages() {
		return anchorages;
	}

	/** {@inheritDoc} */
	@Override
	public IContentMVCBundle getParentBundle() {
		return parent;
	}

	/** {@inheritDoc} */
	@Override
	public IContentVisual getVisual() {
		return (IContentVisual)super.getVisual();
	}

	/** {@inheritDoc} */
	@Override
	public void setParentBundle(IContentMVCBundle p) {
		if(parent != null) {
			parent.getChildren().remove(this);
		}
		this.parent = p;
		if(parent != null) {
			parent.getChildren().add(this);
		}
	}

	/** {@inheritDoc} */
	@Override
	public List<IContentMVCBundleWithParent> getChildren() {
		return children;
	}

	/** {@inheritDoc} */
	@Override
	public void changed(Object element) {
		// Check if the anchorages have changed.
		boolean anchoragesChanged = false;
		List<Object> modelAnchorages = new ArrayList<>(
				getViewer().getModelFactory().getContentAnchorageModels(getModel()));
		for(IContentAnchorageMVCBundle a : getAnchorages()) {
			anchoragesChanged |= !modelAnchorages.remove(a.getModel());
		}
		anchoragesChanged |= modelAnchorages.size() > 0;

		// If the anchorages have changed, invoke the generic case from the superclass
		if(anchoragesChanged) {
			super.changed(element);
			return;
		}

		// Otherwise, only the properties of the element can have changed.
		// Update the view of the element, its anchorages and the connected links.
		getViewer().updateVisual(this);
	}
}
