/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.common.ui.javafx.util;

import java.io.InputStream;
import java.net.URL;
import java.util.HashMap;

import javafx.scene.image.Image;
import javafx.scene.paint.Color;

/**
 * Utility methods for handling graphics in JavaFX-based views.
 * 
 * @author munaro
 */
public class GraphicUtils {
	/** The image cache of loaded images. */
	private static final HashMap<String, Image> imageCache = new HashMap<>();

	/** Returns the plugin-local URI string for the given resource. */
	private static String getURIString(String pluginId, String localPath) {
		if(!pluginId.endsWith("/") && !localPath.startsWith("/")) {
			localPath = "/" + localPath;
		}
		return "platform:/plugin/" + pluginId + localPath;
	}

	/** Returns the Java FX Image loaded from the plugin's local path. */
	public static Image getFXImage(String pluginId, String localPath) {
		if(pluginId == null || localPath == null) {
			return null;
		}
		String uri = getURIString(pluginId, localPath);
		if(uri == null) {
			return null;
		}
		return getFXImageFromURI(uri);
	}

	/** Returns the Java FX Image loaded from the given URI. */
	public static Image getFXImageFromURI(String uri) {
		Image cacheImage = imageCache.get(uri);
		if(cacheImage == null) {
			InputStream in = null;
			try {
				in = new URL(uri).openStream();
			} catch(Exception e) {
				e.printStackTrace();
			}
			if(in != null) {
				cacheImage = new Image(in);
			}
			imageCache.put(uri, cacheImage);
		}
		return cacheImage;
	}

	/** Converts the given SWT color to Java FX color. */
	public static Color convertColor(org.eclipse.swt.graphics.Color swtColor) {
		return Color.rgb(swtColor.getRed(), swtColor.getGreen(), swtColor.getBlue(),
				swtColor.getAlpha() / 255.0);
	}
}
