/*-------------------------------------------------------------------------+
| Copyright 2011 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.extension.base;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;

import javafx.embed.swt.FXCanvas;
import javafx.scene.Parent;
import javafx.scene.Scene;

/**
 * Base implementation of model element editors which are using JavaFX.
 * 
 * @author hoelzlf
 */
public abstract class FXEditorBase<T extends EObject> extends EditorBase<T>
		implements ISelectionProvider {
	/** The list of {@link ISelectionChangedListener}s. */
	private final List<ISelectionChangedListener> selectionListeners = new ArrayList<>();

	/** {@inheritDoc} */
	@Override
	public final void createPartControl(Composite parent) {
		FXCanvas canvas = new FXCanvas(parent, SWT.NONE);
		Scene scene = new Scene(createSceneRoot());
		canvas.setScene(scene);
		getSite().setSelectionProvider(this);
	}

	/** Creates the root node of the scene. */
	protected abstract Parent createSceneRoot();

	/** {@inheritDoc} */
	@Override
	public final void addSelectionChangedListener(ISelectionChangedListener listener) {
		if(!selectionListeners.contains(listener)) {
			selectionListeners.add(listener);
		}
	}

	/** {@inheritDoc} */
	@Override
	public final void removeSelectionChangedListener(ISelectionChangedListener listener) {
		selectionListeners.remove(listener);
	}

	/** {@inheritDoc} */
	@Override
	public final void setSelection(ISelection selection) {
		SelectionChangedEvent evt = new SelectionChangedEvent(this, selection);
		for(ISelectionChangedListener scl : selectionListeners) {
			scl.selectionChanged(evt);
		}
	}

	/** {@inheritDoc} */
	@Override
	public ISelection getSelection() {
		return StructuredSelection.EMPTY;
	}
}
