/*******************************************************************************
 * Copyright (c) 2017, 2018 fortiss GmbH. 
 * 
 * This program and the accompanying materials are made available under the
 * terms of the Apache License, Version 2.0, which is available at 
 * https://www.apache.org/licenses/LICENSE-2.0.
 *
 * SPDX-License-Identifier: Apache-2.0
 *******************************************************************************/
package org.fortiss.tooling.spiderchart.control;

import org.fortiss.tooling.spiderchart.model.SpiderChart;
import org.fortiss.tooling.spiderchart.style.ChartStyle;

import javafx.geometry.Bounds;
import javafx.scene.canvas.Canvas;
import javafx.scene.layout.Pane;

/**
 * Represents a viewer on the canvas to display the spider chart.
 * 
 * @author hoelzl
 */
public final class SpiderChartViewer {
	/** The viewer main pane. */
	private final Pane viewerPane;
	/** The actual canvas to be used for drawing. */
	private final SpiderChartCanvas canvas;

	/** Constructor */
	public SpiderChartViewer(SpiderChart chart, ChartStyle style) {
		canvas = new SpiderChartCanvas(chart, style);
		viewerPane = new Pane();
		viewerPane.getChildren().add(canvas.getCanvasControl());
		viewerPane.widthProperty().addListener(evt -> {
			updateCanvasSize();
		});
		viewerPane.heightProperty().addListener(evt -> {
			updateCanvasSize();
		});
	}

	/** Updates the canvas size after viewer size changed. */
	protected void updateCanvasSize() {
		Bounds b = viewerPane.getLayoutBounds();
		Canvas c = canvas.getCanvasControl();
		c.setWidth(b.getWidth());
		c.setHeight(b.getHeight());
	}

	/** Returns the viewer pane. */
	public Pane getViewerPane() {
		return viewerPane;
	}
}
