/*******************************************************************************
 * Copyright (c) 2017, 2018 fortiss GmbH. 
 * 
 * This program and the accompanying materials are made available under the
 * terms of the Apache License, Version 2.0, which is available at 
 * https://www.apache.org/licenses/LICENSE-2.0.
 *
 * SPDX-License-Identifier: Apache-2.0
 *******************************************************************************/
package org.fortiss.tooling.spiderchart.model;

import static java.util.Collections.sort;

import java.util.Comparator;
import java.util.List;

/**
 * Class representing an axis of the spider chart that displays enumeration values.
 * 
 * @author hoelzl
 */
public final class EnumerationAxis<T extends Object> extends AxisBase {

	/** The enumeration values of this axis. */
	private final List<T> values;

	/** Constructor. */
	public EnumerationAxis(String name, Comparator<T> comparator, List<T> values) {
		super(name);
		this.values = values;
		sort(this.values, comparator);
	}

	/** Returns the member at the given index. */
	public T getEnumerationMemberAtIndex(int index) {
		if(index >= 0 && index < values.size()) {
			return values.get(index);
		}
		return null;
	}

	/** Returns the index of the given member. */
	public int indexOfEnumerationMember(T member) {
		return values.indexOf(member);
	}

	/** {@inheritDoc} */
	@Override
	public boolean isValidValue(Object d) {
		return values.contains(d);
	}
}
