/*******************************************************************************
 * Copyright (c) 2017, 2018 fortiss GmbH. 
 * 
 * This program and the accompanying materials are made available under the
 * terms of the Apache License, Version 2.0, which is available at 
 * https://www.apache.org/licenses/LICENSE-2.0.
 *
 * SPDX-License-Identifier: Apache-2.0
 *******************************************************************************/
package org.fortiss.tooling.spiderchart.style;

import org.fortiss.tooling.common.ui.javafx.style.FontStyle;

/**
 * Class for all style attributes of the spider chart legend.
 * 
 * @author hoelzl
 */
public final class LegendStyle {
	/** The vertical / horizontal layout flag. */
	private boolean verticalLayout;
	/** The inner margins. */
	private double margin = 5;
	/** The font style of the label. */
	private FontStyle labelStyle;

	/** Constructor. */
	public LegendStyle(boolean verticalLayout, double margin, FontStyle labelStyle) {
		this.margin = Math.max(5, margin);
		this.verticalLayout = verticalLayout;
		this.labelStyle = labelStyle;
	}

	/** Returns the vertical layout flag. */
	public boolean isVerticalLayout() {
		return verticalLayout;
	}

	/** Returns margin. */
	public double getMargin() {
		return margin;
	}

	/** Returns the label font style. */
	public FontStyle getLabelStyle() {
		return labelStyle;
	}
}
