/*-------------------------------------------------------------------------+
|                                                                          |
| Copyright 2005-2011 The ConQAT Project                                   |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+-------------------------------------------------------------------------*/
package org.conqat.ide.commons.ui.wizard;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.model.WorkbenchLabelProvider;

/**
 * Wizard page for selecting one project in the workspace.
 * 
 * @author heineman
 * @author $Author: deissenb $
 * @version $Rev: 34252 $
 * @levd.rating GREEN Hash: BCA3D0179A2A726F04EB3301C5C5658A
 */
public class ProjectSelectionWizardPage extends WizardPage {

	/** The table viewer for the projects */
	private TableViewer projectsTableViewer;
	/** The initial selection */
	private final IProject initialSelection;

	/** Constructor */
	public ProjectSelectionWizardPage(IProject initialSelection, String title,
			String message) {
		super(ProjectSelectionWizardPage.class.getName());
		this.initialSelection = initialSelection;
		setTitle(title);
		setMessage(message);
	}

	/** {@inheritDoc} */
	@Override
	public void createControl(Composite parent) {
		Composite composite = new Composite(parent, SWT.NULL);
		composite.setLayout(new GridLayout());
		projectsTableViewer = new TableViewer(composite, SWT.SINGLE
				| SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER);
		projectsTableViewer.setContentProvider(new ArrayContentProvider());
		projectsTableViewer.setLabelProvider(new WorkbenchLabelProvider());
		projectsTableViewer.setInput(ResourcesPlugin.getWorkspace().getRoot()
				.getProjects());
		projectsTableViewer
				.addSelectionChangedListener(new ISelectionChangedListener() {
					@Override
					public void selectionChanged(SelectionChangedEvent event) {
						updateStatus();
					}
				});
		projectsTableViewer.addDoubleClickListener(new IDoubleClickListener() {
			@Override
			public void doubleClick(DoubleClickEvent event) {
				getContainer().showPage(getNextPage());
			}
		});
		GridDataFactory.fillDefaults().hint(500, 300).applyTo(
				projectsTableViewer.getTable());
		setControl(composite);
		setPageComplete(false);
		if (initialSelection != null) {
			projectsTableViewer.setSelection(new StructuredSelection(
					initialSelection));
			projectsTableViewer.reveal(initialSelection);
			setPageComplete(true);
		}
	}

	/** Updates the status of the error message */
	private void updateStatus() {
		setPageComplete(!projectsTableViewer.getSelection().isEmpty());
		if (projectsTableViewer.getSelection().isEmpty()) {
			setErrorMessage("No project selected");
		} else {
			setErrorMessage(null);
		}
	}

	/** Retrieves the selected project */
	public IProject getSelectedProject() {
		return (IProject) ((IStructuredSelection) projectsTableViewer
				.getSelection()).getFirstElement();
	}
}